<?php

namespace SGalinski\SgYoutube\Controller;

/***************************************************************
 *  Copyright notice
 *  (c) sgalinski Internet Services (https://www.sgalinski.de)
 *  All rights reserved
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use Exception;
use SGalinski\SgYoutube\Service\CachedImageService;
use SGalinski\SgYoutube\Service\YoutubeService;
use TYPO3\CMS\Core\Imaging\ImageManipulation\CropVariantCollection;
use TYPO3\CMS\Core\Resource\FileReference;
use TYPO3\CMS\Core\Resource\FileRepository;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Mvc\Controller\ActionController;
use TYPO3\CMS\Extbase\Service\ImageService;

/**
 *  The Youtube Controller
 */
class YoutubeController extends ActionController {
	/**
	 * @var YoutubeService
	 */
	protected $youtubeService;

	/**
	 * Injects the YouTube service. Is lot faster to use the inject method than the inject annotation!
	 *
	 * @param YoutubeService $youtubeService
	 * @return void
	 */
	public function injectYoutubeService(YoutubeService $youtubeService) {
		$this->youtubeService = $youtubeService;
	}

	/**
	 * Renders the YouTube video view
	 *
	 * @return NULL|string
	 */
	public function indexAction() {
		$id = $this->settings['id'];
		$filterIds = $this->settings['filterId'] ?? '';
		$maxResults = $this->settings['maxResults'];
		$apiKey = $this->settings['apiKey'];
		$thumbnailType = $this->settings['thumbnailType'];
		$aspectRatio = $this->settings['aspectRatio'];
		$showApiResult = (bool) $this->settings['showApiResult'];
		$debugOutput = '';

		$filterIds = explode(',', $filterIds);
		foreach ($filterIds as &$filterId) {
			$filterId = trim($filterId, " ");
		}

		$maxResultsWithFilters = (string) (intval($maxResults) + count($filterIds));

		try {
			$jsonArray = $this->youtubeService->getJsonAsArray($id, $maxResultsWithFilters, $apiKey);

			$jsonArray['items'] = array_filter($jsonArray['items'], function ($item) use ($filterIds) {
				$videoId = $item['snippet']['resourceId']['videoId'] ?? $item['id'];
				if (is_array($videoId)) {
					$videoId = $videoId['videoId'];
				}
				foreach ($filterIds as $filterId) {
					if ($videoId === $filterId) {
						return false;
					}
				}
				return true;
			});

			// Fix the array indexes from previous filtering
			$jsonArray['items'] = array_values($jsonArray['items']);

			while (count($jsonArray['items']) > $maxResults) {
				array_pop($jsonArray['items']);
			}

			if ($showApiResult) {
				$debugOutput = nl2br($id . "\n\n" . print_r($jsonArray, TRUE));
			}

			$jsonArray['items'] = $this->youtubeService->mapArray(
				$jsonArray['items'],
				$id,
				$aspectRatio,
				$thumbnailType,
				$apiKey
			);

			$jsonArray['items'] = $this->mapJsonArrayWithPossibleCustomThumbnails($jsonArray['items']);
		} catch (Exception $exception) {
			return '<div style="color: red;">' . $exception->getMessage() . '</div>';
		}

		$cachedImageService = GeneralUtility::makeInstance(CachedImageService::class, 'youtube');
		foreach ($jsonArray['items'] as $key => $item) {
			if (isset($item['thumbnail'])) {
				$jsonArray['items'][$key]['thumbnail'] = $cachedImageService->getImage($item['thumbnail']);
			}
		}

		$this->view->assignMultiple(
			[
				'feed' => $jsonArray['items'],
				'response' => $jsonArray,
				'debugOutput' => $debugOutput,
				'showTitle' => (int) ($this->settings['showTitle'] ?? 1),
				'showDescription' => (int) ($this->settings['showDescription'] ?? 1),
			]
		);

		return NULL;
	}

	/**
	 * Maps the given jsonArray thumbnails with the possible custom ones from the plugin settings.
	 *
	 * @param array $jsonArray
	 * @return array
	 */
	protected function mapJsonArrayWithPossibleCustomThumbnails(array $jsonArray): array {
		$contentElementUid = (int) $this->configurationManager->getContentObject()->data['uid'];
		if ($contentElementUid <= 0) {
			return $jsonArray;
		}

		$fileRepository = GeneralUtility::makeInstance(FileRepository::class);
		$fileObjects = $fileRepository->findByRelation(
			'tt_content',
			'tx_sgyoutube_thumbnail_image',
			$contentElementUid
		);
		if (count($fileObjects) <= 0) {
			return $jsonArray;
		}

		/** @var FileReference $fileObject */
		foreach ($fileObjects as $index => $fileObject) {
			if (!isset($jsonArray[$index])) {
				break;
			}

			$cropString = '';
			if ($fileObject->hasProperty('crop') && $fileObject->getProperty('crop')) {
				$cropString = $fileObject->getProperty('crop');
			}

			$cropVariantCollection = CropVariantCollection::create((string) $cropString);
			$cropArea = $cropVariantCollection->getCropArea('default');
			$processingInstructions = [
				'crop' => $cropArea->isEmpty() ? NULL : $cropArea->makeAbsoluteBasedOnFile($fileObject),
			];
			$imageService = GeneralUtility::makeInstance(ImageService::class);
			$processedImage = $imageService->applyProcessingInstructions($fileObject, $processingInstructions);
			$jsonArray[$index]['thumbnail'] = $imageService->getImageUri($processedImage);
		}

		return $jsonArray;
	}
}
